/*
# X-BASED PYRAMINX(tm)
#
#  PyraminxP.h
#
###
#
#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
#
#                   All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee is hereby granted,
#  provided that the above copyright notice appear in all copies and
#  that both that copyright notice and this permission notice appear in
#  supporting documentation, and that the name of the author not be
#  used in advertising or publicity pertaining to distribution of the
#  software without specific, written prior permission.
#
#  This program is distributed in the hope that it will be "playable",
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
*/

/* Private header file for Pyraminx */

#ifndef _PyraminxP_h
#define _PyraminxP_h

#include "Pyraminx.h"

/*** random number generator ***/
/* insert your favorite */
extern void SetRNG();
extern long LongRNG();
#define SRAND(X) SetRNG((long) X)
#define LRAND() LongRNG()
 
#define NRAND(X) ((int)(LRAND()%(X)))

#define SYMBOL ':'

#define IGNORE (-1)
#define TOP 0
#define TR 1
#define RIGHT 2
#define BOTTOM 3
#define BL 4
#define LEFT 5
#define COORD 6
#define CW 7
#define CCW 11

/* The following are in xpyraminx.c also */
#define MINTETRAS 1
#define PERIOD2 2
#define PERIOD3 3
#define BOTH 4
#define MAXMODES 3

#define DEFAULTMODE PERIOD3
#define DEFAULTTETRAS 3
#define MAXFACES 4
#define DOWN 1
#define UP 0
#define MAXVIEWS 2
#define MAXSIDES (MAXFACES/MAXVIEWS) 
#define MAXORIENT (3*MAXSIDES)

#define ABS(a) (((a)<0)?(-a):(a))
#define SIGN(a) (((a)<0)?(-1):1)
#define MIN(a,b) (((int)(a)<(int)(b))?(int)(a):(int)(b))
#define MAX(a,b) (((int)(a)>(int)(b))?(int)(a):(int)(b))

typedef struct _PyraminxLoc
{
  int face, rotation;
} PyraminxLoc;

typedef struct _PyraminxPart
{
  Pixel foreground;
  Pixel borderColor;
  Pixel faceColor[MAXFACES];
  PyraminxLoc *tetraLoc[MAXFACES];
  PyraminxLoc *faceLoc[MAXSIDES];
  PyraminxLoc *rowLoc[3][MAXSIDES];
  int currentFace, currentPosition;
  Boolean started, practice, orient, vertical, mono, sticky;
  int mode, depth;
  int size, sizeSize;
  Position delta;
  Position tetraLength;
  Position faceLength;
  Position viewLength;
  Position sideOffset, orientLineLength, orientDiagLength;
  XPoint puzzleSize;
  XPoint puzzleOffset;
  GC puzzleGC;
  GC borderGC;
  GC faceGC[MAXFACES];
  GC inverseGC;
  String faceName[MAXFACES];
  XtCallbackList select;
} PyraminxPart;

typedef struct _PyraminxRec
{
  CorePart core;
  PyraminxPart pyraminx;
} PyraminxRec;

/* This gets around C's inability to do inheritance */
typedef struct _PyraminxClassPart
{
  int ignore;
} PyraminxClassPart;

typedef struct _PyraminxClassRec
{
  CoreClassPart core_class;
  PyraminxClassPart pyraminx_class;
} PyraminxClassRec;

extern PyraminxClassRec pyraminxClassRec;
extern PyraminxLoc *startLoc[MAXFACES];

extern void MovePyraminx();
/*extern void SolvePolyhedrons();*/ /* For future auto-solver */
extern void DrawAllPolyhedrons();
extern Boolean CheckSolved();
extern void InitMoves();
extern void PutMove();
extern void GetMove();
extern int MadeMoves();
extern void FlushMoves();
extern int NumMoves();           
extern void ScanMoves();          
extern void PrintMoves();
extern void ScanStartPosition();
extern void PrintStartPosition();
extern void SetStartPosition();

#endif /* _PyraminxP_h */
