/*
# MOTIF-BASED PYRAMINX(tm)
#
#  xmpyraminx.c
#
###
#
#  Copyright (c) 1993 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
#
#                   All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee is hereby granted,
#  provided that the above copyright notice appear in all copies and
#  that both that copyright notice and this permission notice appear in
#  supporting documentation, and that the name of the author not be
#  used in advertising or publicity pertaining to distribution of the
#  software without specific, written prior permission.
#
#  This program is distributed in the hope that it will be "useful",
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
*/

/*
  Version 5: 95/10/04 Xt/Motif
  Version 4: 94/05/31 Xt
  Version 3: 93/04/01 Motif
  Version 2: 92/01/29 XView
  Version 1: 91/03/19 SunView
*/

#include <stdlib.h>
#include <stdio.h>
#ifdef VMS
#include <unixlib.h>
#define getlogin cuserid
#else
#ifndef apollo
#include <unistd.h>
#endif
#endif
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <Xm/PanedW.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Scale.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include "Pyraminx.h"
#include "pyraminx.xbm"
#include "mouse-l.xbm"
#include "mouse-r.xbm"

#ifndef SCOREFILE
#define SCOREFILE "/usr/games/lib/pyraminx.scores"
#endif
 
/* The following are in PyraminxP.h also */
#define MINTETRAS 1
#define PERIOD2 2
#define PERIOD3 3
#define BOTH 4
#define MAXMODES 3

#define MAXTETRAS 7
#define MAXRECORD 32767

static void Initialize();
static void CallbackPyraminx();

static void PrintRecord();
static int HandleSolved();
static void ReadRecords();
static void WriteRecords();
 
static void TetraSlider();
static void ModeToggle();
static void OrientToggle();
static void StickyToggle();
static void PracticeToggle();
static void motif_printf();

static Arg arg[5];
static Widget moves, record, message, pyraminx, modes[MAXMODES],
  orientSwitch, stickySwitch, practiceSwitch, tetras;
static int pyraminxRecord[MAXMODES][2][MAXTETRAS - MINTETRAS + 2];
static int movesDsp = 0;
static int oldSize;
static char messageDsp[128] = "Welcome";

static char *modeString[] = {
  "Period 2", "Period 3", "Both"
};

static void Usage()
{
  (void) fprintf(stderr, "usage: xmpyraminx\n");
  (void) fprintf(stderr,
    "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
  (void) fprintf(stderr,
    "\t[-display [{host}]:[{vs}]] [-fg {color}] [-bg {color}]\n");
  (void) fprintf(stderr,
    "\t[-{border|bd} {color}] [-mono] [-{size {int} | sticky}]\n");
  (void) fprintf(stderr,
    "\t[[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n");
  (void) fprintf(stderr,
    "\t [-face{0|1|2|3} {color}]\n");
  exit(1);
}

static XrmOptionDescRec options[] = {
  {"-fg",		"*pyraminx.Foreground",	XrmoptionSepArg,	NULL},
  {"-bg",		"*Background",		XrmoptionSepArg,	NULL},
  {"-foreground",	"*pyraminx.Foreground",	XrmoptionSepArg,	NULL},
  {"-background",	"*Background",		XrmoptionSepArg,	NULL},
  {"-border",		"*pyraminx.pieceBorder",XrmoptionSepArg,	NULL},
  {"-bd",		"*pyraminx.pieceBorder",XrmoptionSepArg,	NULL},
  {"-mono",		"*pyraminx.mono",	XrmoptionNoArg,		"TRUE"},
  {"-size",		"*pyraminx.size",	XrmoptionSepArg,	NULL},
  {"-sticky",		"*pyraminx.sticky",	XrmoptionNoArg,		"FALSE"},
  {"-mode",		"*pyraminx.mode",	XrmoptionSepArg,	NULL},
  {"-both",		"*pyraminx.mode",	XrmoptionNoArg,		"4"},
  {"-orient",		"*pyraminx.orient",	XrmoptionNoArg,		"TRUE"},
  {"-noorient",		"*pyraminx.orient",	XrmoptionNoArg,		"FALSE"},
  {"-practice",		"*pyraminx.practice",	XrmoptionNoArg,		"TRUE"},
  {"-nopractice",	"*pyraminx.practice",	XrmoptionNoArg,		"FALSE"},
  {"-face0",		"*pyraminx.faceColor0",	XrmoptionSepArg,	NULL},
  {"-face1",		"*pyraminx.faceColor1",	XrmoptionSepArg,	NULL},
  {"-face2",		"*pyraminx.faceColor2",	XrmoptionSepArg,	NULL},
  {"-face3",		"*pyraminx.faceColor3",	XrmoptionSepArg,	NULL}
};

int main(argc, argv)
  int argc;
  char *argv[];
{
  Widget toplevel; 
  Widget panel, panel2, rowcol, rowcol2, rowcol3, rowcol4;
  Pixmap mouseLeftCursor, mouseRightCursor;
  Pixel fg, bg;
  int i;

  toplevel = XtInitialize(argv[0], "Pyraminx",
    options, XtNumber(options), &argc, argv);
  if (argc != 1)
    Usage();

  XtSetArg(arg[0], XtNiconPixmap,
    XCreateBitmapFromData(XtDisplay(toplevel),
      RootWindowOfScreen(XtScreen(toplevel)),
      (char *) pyraminx_bits, pyraminx_width, pyraminx_height));
  XtSetArg(arg[1], XmNkeyboardFocusPolicy, XmPOINTER); /* not XmEXPLICIT */
  XtSetValues(toplevel, arg, 2);
  panel = XtCreateManagedWidget("panel", xmPanedWindowWidgetClass, toplevel,
    NULL, 0);
  panel2 = XtVaCreateManagedWidget("panel2", xmPanedWindowWidgetClass, panel,
    XmNseparatorOn, False,
    XmNsashWidth, 1,
    XmNsashHeight, 1,
    NULL);

  rowcol = XtVaCreateManagedWidget("Rowcol", xmRowColumnWidgetClass, panel2,
    XmNnumColumns, 2,
    XmNorientation, XmHORIZONTAL,
    XmNpacking, XmPACK_COLUMN,
    NULL);
  XtVaGetValues(rowcol, XmNforeground, &fg, XmNbackground, &bg, NULL);
  mouseLeftCursor = XCreatePixmapFromBitmapData(XtDisplay(rowcol),
    RootWindowOfScreen(XtScreen(rowcol)), mouse_left_bits,
    mouse_left_width, mouse_left_height, fg, bg,
    DefaultDepthOfScreen(XtScreen(rowcol)));
  mouseRightCursor = XCreatePixmapFromBitmapData(XtDisplay(rowcol),
    RootWindowOfScreen(XtScreen(rowcol)), mouse_right_bits,
    mouse_right_width, mouse_right_height, fg, bg,
    DefaultDepthOfScreen(XtScreen(rowcol)));
  XtVaCreateManagedWidget("mouseLeftText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Move", 5, NULL);
  XtVaCreateManagedWidget("mouseLeft", xmLabelGadgetClass, rowcol,
    XmNlabelType, XmPIXMAP, XmNlabelPixmap, mouseLeftCursor, NULL);
  XtVaCreateManagedWidget("mouseRightText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Randomize", 10, NULL);
  XtVaCreateManagedWidget("mouseRight", xmLabelGadgetClass, rowcol,
    XmNlabelType, XmPIXMAP, XmNlabelPixmap, mouseRightCursor, NULL);
  XtVaCreateManagedWidget("CallbackPyraminx", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Moves", 6, NULL);
  moves = XtVaCreateManagedWidget("0", xmLabelWidgetClass, rowcol, NULL);
  XtVaCreateManagedWidget("record_text", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
  record = XtVaCreateManagedWidget("0", xmLabelWidgetClass, rowcol, NULL);

  rowcol2 = XtVaCreateManagedWidget("Rowcol2", xmRowColumnWidgetClass, panel2,
    NULL);
  XtVaGetValues(rowcol2, XmNforeground, &fg, XmNbackground, &bg, NULL);
  tetras = XtVaCreateManagedWidget("tetras", xmScaleWidgetClass, rowcol2,
    XtVaTypedArg, XmNtitleString, XmRString, "Tetras", 7,
    XmNminimum, MINTETRAS,
    XmNmaximum, MAXTETRAS,
    XmNvalue, MINTETRAS,
    XmNshowValue, True,
    XmNorientation, XmHORIZONTAL,
    NULL);
  XtAddCallback(tetras, XmNvalueChangedCallback, TetraSlider, NULL);
  rowcol3 = XtVaCreateManagedWidget("Rowcol", xmRowColumnWidgetClass, rowcol2,
    XmNnumColumns, 1,
    XmNorientation, XmHORIZONTAL,
    XmNpacking, XmPACK_COLUMN,
    XmNradioBehavior, True,
    NULL);
  for (i = 0; i < XtNumber(modeString); i++) {
    modes[i] = XtVaCreateManagedWidget(modeString[i],
      xmToggleButtonGadgetClass, rowcol3,
      XmNradioBehavior, True,
      NULL);
    XtAddCallback(modes[i], XmNvalueChangedCallback, ModeToggle,
      (XtPointer) i);
  }
  rowcol4 = XtVaCreateManagedWidget("Rowcol", xmRowColumnWidgetClass, rowcol2,
    XmNnumColumns, 1,
    XmNorientation, XmHORIZONTAL,
    XmNpacking, XmPACK_COLUMN,
    NULL);
  orientSwitch = XtVaCreateManagedWidget ("Orient",
    xmToggleButtonWidgetClass, rowcol4,
    NULL);
  XtAddCallback(orientSwitch, XmNvalueChangedCallback, OrientToggle, NULL);
  stickySwitch = XtVaCreateManagedWidget ("Sticky",
    xmToggleButtonWidgetClass, rowcol4,
    NULL);
  XtAddCallback(stickySwitch, XmNvalueChangedCallback, StickyToggle, NULL);
  practiceSwitch = XtVaCreateManagedWidget ("Practice",
    xmToggleButtonWidgetClass, rowcol4,
    NULL);
  XtAddCallback(practiceSwitch, XmNvalueChangedCallback, PracticeToggle, NULL);
  message = XtVaCreateManagedWidget("Play Pyraminx! (use mouse and keypad)",
    xmLabelWidgetClass, rowcol2,
    NULL);

  pyraminx = XtCreateManagedWidget("pyraminx", pyraminxWidgetClass, panel,
    NULL, 0);
  XtAddCallback(pyraminx, XtNselectCallback, CallbackPyraminx, NULL);
  Initialize(pyraminx);
  XtRealizeWidget(toplevel);
  XGrabButton(XtDisplay(pyraminx), AnyButton, AnyModifier, XtWindow(pyraminx),
    TRUE, ButtonPressMask | ButtonMotionMask | ButtonReleaseMask,
    GrabModeAsync, GrabModeAsync, XtWindow(pyraminx),
    XCreateFontCursor(XtDisplay(pyraminx), XC_crosshair));
  XtMainLoop();

#ifdef VMS
  return 1;
#else
  return 0;
#endif
}

static void Initialize(w)
  Widget w;
{
  int size, mode;
  Boolean orient, sticky, practice;

  XtVaSetValues(w,
    XtNstart, FALSE,
    NULL);
  XtVaGetValues(w,
    XtNsize, &size,
    XtNmode, &mode,
    XtNorient, &orient,
    XtNsticky, &sticky,
    XtNpractice, &practice,
    NULL);
  if (size <= MAXTETRAS)
    XmScaleSetValue(tetras, size);
  XmToggleButtonSetState(modes[mode - PERIOD2], True, False);
  XmToggleButtonSetState(orientSwitch, orient, True);
  XmToggleButtonSetState(stickySwitch, sticky, True);
  XmToggleButtonSetState(practiceSwitch, practice, True);
  ReadRecords();
  PrintRecord(size, mode, orient, sticky, practice);
  oldSize = size;
}

static void CallbackPyraminx(w, clientData, callData)
  Widget w;
  caddr_t clientData;
  pyraminxCallbackStruct *callData;
{
  int size, mode;
  Boolean orient, sticky, practice, start;

  XtVaGetValues(w,
    XtNsize, &size,
    XtNmode, &mode,
    XtNorient, &orient,
    XtNsticky, &sticky,
    XtNpractice, &practice,
    XtNstart, &start,
    NULL);
  (void) strcpy(messageDsp, "");
  switch (callData->reason) {
    case PYRAMINX_RESTORE:
      if (practice)
        motif_printf(record, "practice");
      movesDsp = 0;
      break;
    case PYRAMINX_RESET:
      movesDsp = 0;
      break;
    case PYRAMINX_AMBIGUOUS:
      (void) strcpy(messageDsp, "Ambiguous move");
      break;
    case PYRAMINX_ILLEGAL:
      if (practice || start)
        (void) strcpy(messageDsp, "Illegal move");
      else
        (void) strcpy(messageDsp, "Randomize to start");
      break;
    case PYRAMINX_MOVED:
      movesDsp++;
      XtSetArg(arg[0], XtNstart, TRUE);
      XtSetValues(w, arg, 1);
      break;
    case PYRAMINX_CONTROL:
      return;
    case PYRAMINX_SOLVED:
      if (practice)
        movesDsp = 0;
      else {
        if (HandleSolved(movesDsp, size, mode, orient, sticky))
          (void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
        else
          (void) strcpy(messageDsp, "Solved!");
      }
      XtSetArg(arg[0], XtNstart, FALSE);
      XtSetValues(w, arg, 1);
      break;
    case PYRAMINX_PRACTICE:
      movesDsp = 0;
      practice = !practice;
      if (!practice)
        (void) strcpy(messageDsp, "Randomize to start");
      PrintRecord(size, mode, orient, sticky, practice);
      XtSetArg(arg[0], XtNpractice, practice);
      XtSetArg(arg[1], XtNstart, FALSE);
      XtSetValues(w, arg, 2);
      XmToggleButtonSetState(practiceSwitch, practice, True);
      break;
    case PYRAMINX_RANDOMIZE:
      movesDsp = 0;
      XtSetArg(arg[0], XtNpractice, FALSE);
      XtSetArg(arg[1], XtNstart, FALSE);
      XtSetValues(w, arg, 2);
      break;
    case PYRAMINX_DEC:
      if (!sticky) {
        movesDsp = 0;
        size--;
        oldSize = size;
        PrintRecord(size, mode, orient, sticky, practice);
        XtSetArg(arg[0], XtNsize, size);
        XtSetValues(w, arg, 1);
        if (size <= MAXTETRAS)
          XmScaleSetValue(tetras, size);
      }
      break;
    case PYRAMINX_ORIENT:
      movesDsp = 0;
      orient = !orient;
      PrintRecord(size, mode, orient, sticky, practice);
      XtSetArg(arg[0], XtNorient, orient);
      XtSetValues(w, arg, 1);
      XmToggleButtonSetState(orientSwitch, orient, True);
      break;
    case PYRAMINX_INC:
      if (!sticky) {
        movesDsp = 0;
        size++;
        oldSize = size;
        PrintRecord(size, mode, orient, sticky, practice);
        XtSetArg(arg[0], XtNsize, size);
        XtSetValues(w, arg, 1);
        if (size <= MAXTETRAS)
          XmScaleSetValue(tetras, size);
      }
      break;
    case PYRAMINX_PERIOD2:
    case PYRAMINX_PERIOD3:
    case PYRAMINX_BOTH:
      movesDsp = 0;
      mode = callData->reason - PYRAMINX_PERIOD2 + PERIOD2;
      PrintRecord(size, mode, orient, sticky, practice);
      XtSetArg(arg[0], XtNmode, mode);
      XtSetValues(w, arg, 1);
      XmToggleButtonSetState(modes[mode - PERIOD2], True, True);
      break;
    case PYRAMINX_STICKY:
      movesDsp = 0;
      sticky = !sticky;
      if (sticky)
        size = 4;
      else
        size = oldSize;
      PrintRecord(size, mode, orient, sticky, practice);
      XtSetArg(arg[0], XtNsticky, sticky);
      XtSetArg(arg[1], XtNsize, size);
      XtSetValues(w, arg, 2);
      XmToggleButtonSetState(stickySwitch, sticky, True);
      break;
    case PYRAMINX_COMPUTED:
      XtSetArg(arg[0], XtNstart, FALSE);
      XtSetValues(w, arg, 1);
      break;
    case PYRAMINX_UNDO:
      movesDsp--;
      XtSetArg(arg[0], XtNstart, TRUE);
      XtSetValues(w, arg, 1);
      break;
  }
  motif_printf(message, "%s", messageDsp);
  motif_printf(moves, "%d", movesDsp);
}

static void TetraSlider(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmScaleCallbackStruct *cbs;
{
  int size = cbs->value, mode;
  Boolean orient, sticky, practice;
 
  XtVaGetValues(pyraminx,
    XtNsize, &oldSize,
    XtNmode, &mode,
    XtNorient, &orient,
    XtNsticky, &sticky,
    XtNpractice, &practice,
    NULL);
  if (sticky)
    XmScaleSetValue(w, oldSize);
  else if (oldSize != size) {
    XtVaSetValues(pyraminx,
      XtNsize, size,
      NULL);
    oldSize = size;
    movesDsp = 0;
    motif_printf(moves, "%d", movesDsp);
    PrintRecord(size, mode, orient, sticky, practice);
  }
}

static void ModeToggle(w, mode, cbs)
  Widget w;
  int mode;
  XmToggleButtonCallbackStruct *cbs;
{
  int size;
  Boolean orient, sticky, practice;

  if (cbs->set) {
    XtVaGetValues(pyraminx,
      XtNsize, &size,
      XtNorient, &orient,
      XtNsticky, &sticky,
      XtNpractice, &practice,
      NULL);
    XtVaSetValues(pyraminx,
      XtNmode, mode + PERIOD2,
      NULL);
    movesDsp = 0;
    motif_printf(moves, "%d", movesDsp);
    PrintRecord(size, mode + PERIOD2, orient, sticky, practice);
  }
}

static void OrientToggle(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmToggleButtonCallbackStruct *cbs;
{
  int size, mode;
  Boolean orient = cbs->set, sticky, practice;

  XtVaGetValues(pyraminx,
    XtNsize, &size,
    XtNmode, &mode,
    XtNsticky, &sticky,
    XtNpractice, &practice,
    NULL);
  XtVaSetValues(pyraminx,
    XtNorient, orient,
    NULL);
  movesDsp = 0;
  motif_printf(moves, "%d", movesDsp);
  PrintRecord(size, mode, orient, sticky, practice);
}

static void StickyToggle(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmToggleButtonCallbackStruct *cbs;
{
  int size, mode;
  Boolean orient, sticky = cbs->set, practice;

  XtVaGetValues(pyraminx,
    XtNsize, &size,
    XtNmode, &mode,
    XtNorient, &orient,
    XtNpractice, &practice,
    NULL);
  XtVaSetValues(pyraminx,
    XtNsticky, sticky,
    NULL);
  if ((sticky && size != 4) || (!sticky && oldSize != 4)) {
    if (sticky)
      size = 4;
    else
      size = oldSize;
    if (size <= MAXTETRAS)
      XmScaleSetValue(tetras, size);
    XtVaSetValues(pyraminx,
      XtNsize, size,
      NULL);
  }
  movesDsp = 0;
  motif_printf(moves, "%d", movesDsp);
  PrintRecord(size, mode, orient, sticky, practice);
}

static void PracticeToggle(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmToggleButtonCallbackStruct *cbs;
{
  int size, mode;
  Boolean orient, sticky, practice = cbs->set;

  XtVaSetValues(pyraminx,
    XtNpractice, practice,
    XtNstart, FALSE,
    NULL);
  XtVaGetValues(pyraminx,
    XtNsize, &size,
    XtNmode, &mode,
    XtNorient, &orient,
    XtNsticky, &sticky,
    NULL);
  movesDsp = 0;
  motif_printf(moves, "%d", movesDsp);
  if (!practice)
    (void) strcpy(messageDsp, "Randomize to start");
  PrintRecord(size, mode, orient, sticky, practice);
}

static void PrintRecord(size, mode, orient, sticky, practice)
  int size, mode;
  Boolean orient, sticky, practice;
{
  int i = mode - PERIOD2;
  int j = (orient) ? 1 : 0;
  int k = (sticky) ? MAXTETRAS - MINTETRAS + 1 : size - MINTETRAS;

  if (practice)
    motif_printf(record, "practice");
  else if (!sticky && size > MAXTETRAS)
    motif_printf(record, "NOT RECORDED");
  else if (pyraminxRecord[i][j][k] >= MAXRECORD)
    motif_printf(record, "NEVER");
  else
    motif_printf(record, "%d", pyraminxRecord[i][j][k]);
}

static int HandleSolved(counter, size, mode, orient, sticky)
  int counter, size, mode;
  Boolean orient, sticky;
{
  int i = mode - PERIOD2;
  int j = (orient) ? 1 : 0;
  int k = (sticky) ? MAXTETRAS - MINTETRAS + 1 : size - MINTETRAS;

  if ((sticky || size <= MAXTETRAS) && counter < pyraminxRecord[i][j][k]) {
    pyraminxRecord[i][j][k] = counter;
    if (size < 4 || mode == PERIOD2 || (orient &&
        (pyraminxRecord[i][j][k] < pyraminxRecord[i][!j][k])))
      pyraminxRecord[i][!j][k] = counter;
    WriteRecords();
    PrintRecord(size, mode, orient, sticky, FALSE);
    return TRUE;
  }
  return FALSE;
}

static void ReadRecords()
{
  FILE *fp;
  int i, n, mode, orient;
 
  for (mode = 0; mode < MAXMODES; mode++)
    for (orient = 0; orient < 2; orient++)
      for (i = 0; i <= MAXTETRAS - MINTETRAS + 1; i++)
        pyraminxRecord[mode][orient][i] = MAXRECORD;
  if ((fp = fopen(SCOREFILE, "r")) == NULL)
    motif_printf(message, "Can not open %s, taking defaults.", SCOREFILE);
  else {
    for (mode = 0; mode < MAXMODES; mode++)
      for (orient = 0; orient < 2; orient++)
        for (i = 0; i <= MAXTETRAS - MINTETRAS + 1; i++) {
          (void) fscanf(fp, "%d", &n);
          pyraminxRecord[mode][orient][i] = n;
        }
    (void) fclose(fp);
  }
}

static void WriteRecords()
{
  FILE *fp;
  int i, mode, orient;

  if ((fp = fopen(SCOREFILE, "w")) == NULL)
    motif_printf(message, "Can not write to %s.", SCOREFILE);
  else {
    for (mode = 0; mode < MAXMODES; mode++) {
      for (orient = 0; orient < 2; orient++) {
        for (i = 0; i <= MAXTETRAS - MINTETRAS + 1; i++)
          (void) fprintf(fp, "%d ", pyraminxRecord[mode][orient][i]);
        (void) fprintf(fp, "\n");
      }
      (void) fprintf(fp, "\n");
    }
    (void) fclose(fp);
  }
}

#include <varargs.h>
static void motif_printf(va_alist)
  va_dcl
{
  Widget w;
  char *format;
  va_list args;
  char str[1000];
  Arg wargs[10];
  XmString xmstr;

  va_start(args);
  w = va_arg(args, Widget);
  if (!XtIsSubclass(w, xmLabelWidgetClass))
    XtError("motif_printf() requires a Label Widget");
  format = va_arg(args, char *);
  (void) vsprintf(str, format, args);
  xmstr = XmStringCreateLtoR(str, XmSTRING_DEFAULT_CHARSET);
  XtSetArg(wargs[0], XmNlabelString, xmstr);
  XtSetValues(w, wargs, 1);
  va_end(args);
}
