/*
# X-BASED SKEWB
#
#  SkewbP.h
#
###
#
#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
#
#                   All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee is hereby granted,
#  provided that the above copyright notice appear in all copies and
#  that both that copyright notice and this permission notice appear in
#  supporting documentation, and that the name of the author not be
#  used in advertising or publicity pertaining to distribution of the
#  software without specific, written prior permission.
#
#  This program is distributed in the hope that it will be "playable",
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
*/

/* Private header file for Skewb */

#ifndef _SkewbP_h
#define _SkewbP_h

#include "Skewb.h"

/*** random number generator ***/
/* insert your favorite */
extern void SetRNG();
extern long LongRNG();
#define SRAND(X) SetRNG((long) X)
#define LRAND() LongRNG()
 
#define NRAND(X) ((int)(LRAND()%(X)))

#define SYMBOL ':'

#define IGNORE (-1)
#define TR 0
#define BR 1
#define BL 2
#define TL 3
#define STRT 4
#define CW 5
#define HALF 6
#define CCW 7
#define TOP 8
#define RIGHT 9
#define BOTTOM 10
#define LEFT 11
#define MAXORIENT 4
#define MAXROTATE 3
#define MAXCUBES (MAXORIENT+1)
#define MINOR 0
#define MAJOR 1

/* The following is in xskewb.c also */
#define MAXFACES 6

#define MIN(a,b) (((int)(a)<(int)(b))?(int)(a):(int)(b))
#define MAX(a,b) (((int)(a)>(int)(b))?(int)(a):(int)(b))

typedef struct _SkewbLoc
{
  int face, rotation;
} SkewbLoc;

typedef struct _SkewbLocPos
{
  int face, position, direction;
} SkewbLocPos;

typedef struct _SkewbPart
{
  Pixel foreground;
  Pixel borderColor;
  Pixel faceColor[MAXFACES];
  SkewbLoc cubeLoc[MAXFACES][MAXCUBES];
  SkewbLoc faceLoc[MAXCUBES];
  SkewbLoc rowLoc[MAXORIENT][MAXCUBES];
  SkewbLoc minorLoc[MAXORIENT], majorLoc[MAXORIENT][MAXORIENT];
  int currentFace, currentPosition, currentDirection;
  Boolean started, practice, orient, vertical, mono;
  int dim; /* This allows us to reuse code between 2d and 3d */
  int depth;
  Position delta;
  Position orientLineLength;
  XPoint puzzleSize;
  XPoint puzzleOffset, letterOffset;
  GC puzzleGC;
  GC borderGC;
  GC faceGC[MAXFACES];
  GC inverseGC;
  String faceName[MAXFACES];
  XtCallbackList select;
  SkewbLoc skewbLoc[MAXFACES][MAXCUBES];
} SkewbPart;

typedef struct _SkewbRec
{
  CorePart core;
  SkewbPart skewb;
} SkewbRec;

/* This gets around C's inability to do inheritance */
typedef struct _SkewbClassPart
{
  int ignore;
} SkewbClassPart;

typedef struct _SkewbClassRec
{
  CoreClassPart core_class;
  SkewbClassPart skewb_class;
} SkewbClassRec;

typedef struct _RowNext
{
  int face, direction, sideFace;
} RowNext;

extern SkewbClassRec skewbClassRec;

extern void QuitSkewb();
extern void PracticeSkewb();
extern void PracticeSkewbMaybe();
extern void RandomizeSkewb();
extern void RandomizeSkewbMaybe();
extern void GetSkewb();
extern void WriteSkewb();
extern void UndoSkewb();
extern void SolveSkewb();
extern void OrientizeSkewb();
extern void MoveSkewbCw();
extern void MoveSkewbCcw();
extern void MoveSkewbInput();
extern void MoveSkewb();
extern void SelectSkewb();
extern void ReleaseSkewb();
/*extern void SolvePolyhedrons();*/
extern void DrawAllPolyhedrons();
extern Boolean CheckSolved();
extern void InitMoves();
extern void PutMove();
extern void GetMove();
extern int MadeMoves();
extern void FlushMoves();
extern int NumMoves();
extern void ScanMoves();
extern void PrintMoves();
extern void ScanStartPosition();
extern void PrintStartPosition();
extern void SetStartPosition();

#ifdef DEBUG
extern void PrintCube();
#endif

#endif /* _SkewbP_h */
