/*
# MOTIF-BASED SKEWB
#
#  xmskewb.c
#
###
#
#  Copyright (c) 1993 - 95	David Albert Bagley, bagleyd@hertz.njit.edu
#
#                   All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee is hereby granted,
#  provided that the above copyright notice appear in all copies and
#  that both that copyright notice and this permission notice appear in
#  supporting documentation, and that the name of the author not be
#  used in advertising or publicity pertaining to distribution of the
#  software without specific, written prior permission.
#
#  This program is distributed in the hope that it will be "useful",
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
*/

/*
  Version 5: 95/10/06 Xt/Motif
  Version 4: 94/05/30 Xt
  Version 3: 93/10/03 Motif
  Version 2: 92/01/22 XView
  Version 1: 91/03/19 SunView
*/

#include <stdlib.h>
#include <stdio.h>
#ifdef VMS
#include <unixlib.h>
#define getlogin cuserid
#else
#ifndef apollo
#include <unistd.h>
#endif
#endif
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <Xm/PanedW.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleB.h>
#include "Skewb.h"
#include "Skewb2d.h"
#include "Skewb3d.h"
#include "skewb.xbm"
#include "mouse-l.xbm"
#include "mouse-r.xbm"

#ifndef SCOREFILE
#define SCOREFILE "/usr/games/lib/mball.scores"
#endif

/* The following is in SkewbP.h also */
#define MAXFACES 6

#define MAXRECORD 32767

static void Initialize();
static void CallbackSkewb();

static void PrintRecord();
static int HandleSolved();
static void ReadRecords();
static void WriteRecords();

static void OrientToggle();
static void PracticeToggle();
static void motif_printf();

static Arg arg[4];
static Widget moves, record, message, skewb2d, skewb3d,
  orientSwitch, practiceSwitch;
static int skewbRecord[2], movesDsp = 0;
static char messageDsp[128] = "Welcome";

static void Usage()
{
  (void) fprintf(stderr, "usage: xmskewb\n");
  (void) fprintf(stderr,
    "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
  (void) fprintf(stderr,
    "\t[-display [{host}]:[{vs}]][-fg {color}] [-bg {color}]\n");
  (void) fprintf(stderr,
    "\t[-{border|bd} {color}] [-mono] [-[no]orient]\n");
  (void) fprintf(stderr,
    "\t-[no]practice] [-face{0|1|2|3|4|5} {color}]\n");
  exit(1);
}

static XrmOptionDescRec options[] = {
  {"-fg",		"*skewb.Foreground",	XrmoptionSepArg,	NULL},
  {"-bg",		"*Background",		XrmoptionSepArg,	NULL},
  {"-foreground",	"*skewb.Foreground",	XrmoptionSepArg,	NULL},
  {"-background",	"*Background",		XrmoptionSepArg,	NULL},
  {"-border",		"*skewb.pieceBorder",	XrmoptionSepArg,	NULL},
  {"-bd",		"*skewb.pieceBorder",	XrmoptionSepArg,	NULL},
  {"-mono",		"*skewb.mono",		XrmoptionNoArg,		"TRUE"},
  {"-orient",		"*skewb.orient",	XrmoptionNoArg,		"TRUE"},
  {"-noorient",		"*skewb.orient",	XrmoptionNoArg,		"FALSE"},
  {"-practice",		"*skewb.practice",	XrmoptionNoArg,		"TRUE"},
  {"-nopractice",	"*skewb.practice",	XrmoptionNoArg,		"FALSE"},
  {"-face0",		"*skewb.faceColor0",	XrmoptionSepArg,	NULL},
  {"-face1",		"*skewb.faceColor1",	XrmoptionSepArg,	NULL},
  {"-face2",		"*skewb.faceColor2",	XrmoptionSepArg,	NULL},
  {"-face3",		"*skewb.faceColor3",	XrmoptionSepArg,	NULL},
  {"-face4",		"*skewb.faceColor4",	XrmoptionSepArg,	NULL},
  {"-face5",		"*skewb.faceColor5",	XrmoptionSepArg,	NULL},
};

int main(argc, argv)
  int argc;
  char *argv[];
{
  Widget toplevel;
  Widget panel, panel2, rowcol, rowcol2, rowcol3;
  Pixmap mouseLeftCursor, mouseRightCursor;
  Pixel fg, bg;

  toplevel = XtInitialize(argv[0], "Skewb",
    options, XtNumber(options), &argc, argv);
  if (argc != 1)
    Usage();

  XtSetArg(arg[0], XtNiconPixmap,
    XCreateBitmapFromData(XtDisplay(toplevel),
      RootWindowOfScreen(XtScreen(toplevel)),
      (char *) skewb_bits, skewb_width, skewb_height));
  XtSetArg(arg[1], XmNkeyboardFocusPolicy, XmPOINTER); /* not XmEXPLICIT */
  XtSetValues(toplevel, arg, 2);
  panel = XtCreateManagedWidget("panel", xmPanedWindowWidgetClass, toplevel,
    NULL, 0);
  panel2 = XtVaCreateManagedWidget("panel2", xmPanedWindowWidgetClass, panel,
    XmNseparatorOn, False,
    XmNsashWidth, 1,
    XmNsashHeight, 1,
    NULL);

  rowcol = XtVaCreateManagedWidget("Rowcol", xmRowColumnWidgetClass, panel2,
    XmNnumColumns, 2,
    XmNorientation, XmHORIZONTAL,
    XmNpacking, XmPACK_COLUMN,
    NULL);
  XtVaGetValues(rowcol, XmNforeground, &fg, XmNbackground, &bg, NULL);
  mouseLeftCursor = XCreatePixmapFromBitmapData(XtDisplay(rowcol),
    RootWindowOfScreen(XtScreen(rowcol)), mouse_left_bits,
    mouse_left_width, mouse_left_height, fg, bg,
    DefaultDepthOfScreen(XtScreen(rowcol)));
  mouseRightCursor = XCreatePixmapFromBitmapData(XtDisplay(rowcol),
    RootWindowOfScreen(XtScreen(rowcol)), mouse_right_bits,
    mouse_right_width, mouse_right_height, fg, bg,
    DefaultDepthOfScreen(XtScreen(rowcol)));
  XtVaCreateManagedWidget("mouseLeftText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Moves", 5, NULL);
  XtVaCreateManagedWidget("mouseLeft", xmLabelGadgetClass, rowcol,
    XmNlabelType, XmPIXMAP, XmNlabelPixmap, mouseLeftCursor, NULL);
  XtVaCreateManagedWidget("mouseRightText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Randomize", 10, NULL);
  XtVaCreateManagedWidget("mouseRight", xmLabelGadgetClass, rowcol,
    XmNlabelType, XmPIXMAP, XmNlabelPixmap, mouseRightCursor, NULL);
  XtVaCreateManagedWidget("movesText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Moves", 6, NULL);
  moves = XtVaCreateManagedWidget("0", xmLabelWidgetClass, rowcol, NULL);
  XtVaCreateManagedWidget("recordText", xmLabelGadgetClass, rowcol,
    XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
  record = XtVaCreateManagedWidget("0", xmLabelWidgetClass, rowcol, NULL);

  rowcol2 = XtVaCreateManagedWidget("Rowcol2", xmRowColumnWidgetClass, panel2,
    XmNnumColumns, 1,
    XmNorientation, XmHORIZONTAL,
    XmNpacking, XmPACK_COLUMN,
    NULL);
  orientSwitch = XtVaCreateManagedWidget ("Orient",
    xmToggleButtonWidgetClass, rowcol2,
    NULL);
  XtAddCallback(orientSwitch, XmNvalueChangedCallback, OrientToggle, NULL);
  practiceSwitch = XtVaCreateManagedWidget ("Practice",
    xmToggleButtonWidgetClass, rowcol2,
    NULL);
  XtAddCallback(practiceSwitch, XmNvalueChangedCallback, PracticeToggle, NULL);
  rowcol3 = XtVaCreateManagedWidget("Rowcol3", xmRowColumnWidgetClass, panel2,
    NULL);
  message = XtVaCreateManagedWidget("Play Skewb! (use mouse and keypad)",
    xmLabelWidgetClass, rowcol3,
    NULL);

  skewb2d = XtCreateManagedWidget("skewb", skewb2dWidgetClass, panel,
    NULL, 0);
  XtVaSetValues(skewb2d,
    XtNheight, 200,
    NULL);
  XtAddCallback(skewb2d, XtNselectCallback, CallbackSkewb, NULL);
  skewb3d = XtCreateManagedWidget("skewb", skewb3dWidgetClass, panel,
    NULL, 0);
  XtVaSetValues(skewb3d,
    XtNheight, 200,
    NULL);
  XtAddCallback(skewb3d, XtNselectCallback, CallbackSkewb, NULL);
  Initialize();
  XtRealizeWidget(toplevel);
  XGrabButton(XtDisplay(skewb2d), AnyButton, AnyModifier, XtWindow(skewb2d),
    TRUE, ButtonPressMask | ButtonMotionMask | ButtonReleaseMask,
    GrabModeAsync, GrabModeAsync, XtWindow(skewb2d),
    XCreateFontCursor(XtDisplay(skewb2d), XC_crosshair));
  XGrabButton(XtDisplay(skewb3d), AnyButton, AnyModifier, XtWindow(skewb3d),
    TRUE, ButtonPressMask | ButtonMotionMask | ButtonReleaseMask,
    GrabModeAsync, GrabModeAsync, XtWindow(skewb3d),
    XCreateFontCursor(XtDisplay(skewb3d), XC_crosshair));
  XtMainLoop();

#ifdef VMS
  return 1;
#else
  return 0;
#endif
}

/* There's probably a better way to assure that they are the same but
   I don't know it off hand. */
static void MakeEquivalent(orient, practice)
  Boolean *orient, *practice;
{
  Boolean mono;
  Pixel foreground, background, pieceBorder;
  String faceColor[MAXFACES];

  XtVaGetValues(skewb2d,
    XtNorient, orient,
    XtNpractice, practice,
    XtNmono, &mono,
    XtNforeground, &foreground,
    XtNbackground, &background,
    XtNpieceBorder, &pieceBorder,
    XtNfaceColor0, &(faceColor[0]),
    XtNfaceColor1, &(faceColor[1]),
    XtNfaceColor2, &(faceColor[2]),
    XtNfaceColor3, &(faceColor[3]),
    XtNfaceColor4, &(faceColor[4]),
    XtNfaceColor5, &(faceColor[5]),
    NULL);
  XtVaSetValues(skewb2d,
    XtNdirection, SKEWB_IGNORE,
    XtNstart, FALSE,
    NULL);
  XtVaSetValues(skewb3d,
    XtNorient, *orient,
    XtNpractice, *practice,
    XtNmono, mono,
    XtNdirection, SKEWB_IGNORE,
    XtNstart, FALSE,
    XtNforeground, foreground,
    XtNbackground, background,
    XtNpieceBorder, pieceBorder,
    XtNfaceColor0, faceColor[0],
    XtNfaceColor1, faceColor[1],
    XtNfaceColor2, faceColor[2],
    XtNfaceColor3, faceColor[3],
    XtNfaceColor4, faceColor[4],
    XtNfaceColor5, faceColor[5],
    NULL);
}

static void Initialize()
{
  Boolean orient, practice;

  MakeEquivalent(&orient, &practice);
  XmToggleButtonSetState(orientSwitch, orient, True);
  XmToggleButtonSetState(practiceSwitch, practice, True);
  ReadRecords();
  PrintRecord(orient, practice);
}

static void CallbackSkewb(w, clientData, callData)
  Widget w;
  caddr_t clientData;
  skewbCallbackStruct *callData;
{
  Boolean orient, practice, start;
  Widget otherw;

  if (w == skewb2d)
    otherw = skewb3d;
  else /* (w == skewb3d) */
    otherw = skewb2d;
  XtVaGetValues(w,
    XtNorient, &orient,
    XtNpractice, &practice,
    XtNstart, &start,
    NULL);
  (void) strcpy(messageDsp, "");
  switch (callData->reason) {
    case SKEWB_RESTORE:
      XtSetArg(arg[0], XtNdirection, SKEWB_RESTORE);
      XtSetValues(otherw, arg, 1);
      XtSetValues(w, arg, 1);
      movesDsp = 0;
      break;
    case SKEWB_RESET:
      movesDsp = 0;
      break;
    case SKEWB_ILLEGAL:
      if (practice || start)
        (void) strcpy(messageDsp, "Illegal move");
      else
        (void) strcpy(messageDsp, "Randomize to start");
      break;
    case SKEWB_MOVED:
      movesDsp++;
      XtSetArg(arg[0], XtNstart, TRUE);
      XtSetArg(arg[1], XtNface, callData->face);
      XtSetArg(arg[2], XtNpos, callData->position);
      XtSetArg(arg[3], XtNdirection, callData->direction);
      XtSetValues(otherw, arg, 4);
      XtSetValues(w, arg, 1);
      break;
    case SKEWB_CONTROL:
      XtSetArg(arg[0], XtNface, callData->face);
      XtSetArg(arg[1], XtNpos, callData->position);
      XtSetArg(arg[2], XtNdirection, callData->direction);
      XtSetValues(otherw, arg, 3);
      return;
    case SKEWB_SOLVED:
      if (practice)
        movesDsp = 0;
      else { 
        if (HandleSolved(movesDsp, orient))
          (void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
        else
          (void) strcpy(messageDsp, "Solved!");
      }
      XtSetArg(arg[0], XtNstart, FALSE);
      XtSetValues(w, arg, 1);
      XtSetValues(otherw, arg, 1);
      break;
    case SKEWB_PRACTICE:
      movesDsp = 0;
      practice = !practice;
      if (!practice)
        (void) strcpy(messageDsp, "Randomize to start");
      PrintRecord(orient, practice);
      XtSetArg(arg[0], XtNpractice, practice);
      XtSetArg(arg[1], XtNstart, FALSE);
      XtSetValues(w, arg, 2);
      XtSetValues(otherw, arg, 2);
      XmToggleButtonSetState(practiceSwitch, practice, True);
      break;
    case SKEWB_RANDOMIZE:
      movesDsp = 0;
      XtSetArg(arg[0], XtNpractice, FALSE);
      XtSetArg(arg[1], XtNstart, FALSE);
      XtSetValues(w, arg, 2);
      XtSetValues(otherw, arg, 2);
      break; 
    case SKEWB_ORIENT:
      movesDsp = 0;
      orient = !orient;
      PrintRecord(orient, practice);
      XtSetArg(arg[0], XtNorient, orient);
      XtSetValues(w, arg, 1);
      XtSetValues(otherw, arg, 1);
      XmToggleButtonSetState(orientSwitch, orient, True);
      break;
    case SKEWB_COMPUTED:
      XtSetArg(arg[0], XtNstart, FALSE);
      XtSetValues(w, arg, 1);
      XtSetValues(otherw, arg, 1);
      break;
    case SKEWB_UNDO:
      movesDsp--;
      XtSetArg(arg[0], XtNstart, TRUE);
      XtSetArg(arg[1], XtNface, callData->face);
      XtSetArg(arg[2], XtNpos, callData->position);
      XtSetArg(arg[3], XtNdirection, callData->direction);
      XtSetValues(otherw, arg, 4);
      XtSetValues(w, arg, 1);
      break;
  }
  motif_printf(message, "%s", messageDsp);
  motif_printf(moves, "%d", movesDsp);
}

static void OrientToggle(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmToggleButtonCallbackStruct *cbs;
{
  Boolean orient = cbs->set, practice;

  XtVaGetValues(skewb2d,
    XtNpractice, &practice,
    NULL);
  XtVaSetValues(skewb2d,
    XtNorient, orient,
    NULL);
  XtVaSetValues(skewb3d,
    XtNorient, orient,
    NULL);
  movesDsp = 0;
  motif_printf(moves, "%d", movesDsp);
  PrintRecord(orient, practice);
}

static void PracticeToggle(w, clientData, cbs)
  Widget w;
  XtPointer clientData;
  XmToggleButtonCallbackStruct *cbs;
{
  Boolean orient, practice = cbs->set;

  XtVaSetValues(skewb2d,
    XtNpractice, practice,
    XtNstart, FALSE,
    NULL);
  XtVaSetValues(skewb3d,
    XtNpractice, practice,
    XtNstart, FALSE,
    NULL);
  XtVaGetValues(skewb2d,
    XtNpractice, &orient,
    NULL);
  movesDsp = 0;
  motif_printf(moves, "%d", movesDsp);
  if (!practice)
    (void) strcpy(messageDsp, "Randomize to start");
  PrintRecord(orient, practice);
}

static void PrintRecord(orient, practice)
  Boolean orient, practice;
{
  int i = (orient) ? 1 : 0;

  if (practice)
    motif_printf(record, "practice");
  else
  if (skewbRecord[i] >= MAXRECORD)
    motif_printf(record, "NEVER");
  else
    motif_printf(record, "%d", skewbRecord[i]);
}

static int HandleSolved(counter, orient)
  int counter;
  Boolean orient;
{
  int i = (orient) ? 1 : 0;

  if (counter < skewbRecord[i]) {
    skewbRecord[i] = counter;
    if (orient && (skewbRecord[i] < skewbRecord[!i]))
      skewbRecord[!i] = counter;
    WriteRecords();
    PrintRecord(orient, FALSE);
    return TRUE;
  }
  return FALSE;
}

static void ReadRecords()
{
  FILE *fp;
  int n, orient;

  for (orient = 0; orient < 2; orient++)
    skewbRecord[orient] = MAXRECORD;
  if ((fp = fopen(SCOREFILE, "r")) == NULL)
    motif_printf(message, "Can not open %s, taking defaults.", SCOREFILE);
  else {
    for (orient = 0; orient < 2; orient++) {
      (void) fscanf(fp, "%d", &n);
      skewbRecord[orient] = n;
    }
    (void) fclose(fp);
  }
}

static void WriteRecords()
{
  FILE *fp;
  int orient;

  if ((fp = fopen(SCOREFILE, "w")) == NULL)
    motif_printf(message, "Can not write to %s.", SCOREFILE);
  else {
    for (orient = 0; orient < 2; orient++)
      (void) fprintf(fp, "%d ", skewbRecord[orient]);
    (void) fprintf(fp, "\n");
    (void) fclose(fp);
  }
}

#include <varargs.h>
static void motif_printf(va_alist)
  va_dcl
{
  Widget w;
  char *format;
  va_list args;
  char str[1000];
  Arg wargs[10];
  XmString xmstr;

  va_start(args);
  w = va_arg(args, Widget);
  if (!XtIsSubclass(w, xmLabelWidgetClass))
    XtError("motif_printf() requires a Label Widget");
  format = va_arg(args, char *);
  (void) vsprintf(str, format, args);
  xmstr = XmStringCreateLtoR(str, XmSTRING_DEFAULT_CHARSET);
  XtSetArg(wargs[0], XmNlabelString, xmstr);
  XtSetValues(w, wargs, 1);
  va_end(args);
}
